@@ -1,4 +1,4 @@
-;;; package-lint-test.el --- package-lint test suite
+;;; package-lint-test.el --- Test suite for package-lint
 
 ;; Copyright (C) 2016-2017  Steve Purcell, Fanael Linithien
 
@@ -6,6 +6,7 @@
 ;;         Fanael Linithien <fanael4@gmail.com>
 ;; URL: https://github.com/purcell/package-lint
 ;; Version: 0
+;; Package-Requires: ((emacs "24.1"))
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -29,7 +30,8 @@
 (require 'ert)
 
 (defun package-lint-test-add-package-lint-foobar-to-archive (version &optional archive)
-  "Add a package-lint-foobar package to melpa-stable archive.
+  "Add a package-lint-foobar package to ARCHIVE.
+ARCHIVE defaults to \"melpa-stable\".
 VERSION is a list of numbers, e.g., (0 5 0) to represent version
 0.5.0."
   (package--add-to-archive-contents
@@ -38,23 +40,28 @@ VERSION is a list of numbers, e.g., (0 5 0) to represent version
                                       (:url . "https://gitlab.com/somewhere"))])
    (or archive "melpa-stable")))
 
-(defun package-lint-test--run (contents &optional header version footer provide commentary url)
+(defun package-lint-test--run (contents &optional header version footer provide commentary url featurename)
   "Run `package-lint-buffer' on a temporary buffer with given CONTENTS.
 
 HEADER, VERSION, FOOTER, PROVIDE, COMMENTARY, and URL can be
 either strings or nil; when one is a string, the corresponding
 package boilerplate part is replaced with the passed string, when
-it's nil, the default is used."
+it's nil, the default is used.
+
+FEATURENAME defaults to \"test\", and is used in the file name,
+headers and provide form."
+  (unless featurename
+    (setq featurename "test"))
   (with-temp-buffer
     (emacs-lisp-mode)
-    (insert (or header ";;; test.el --- A test\n"))
+    (insert (or header (format ";;; %s.el --- A test\n" featurename)))
     (insert (or version ";; Package-Version: 0\n"))
     (insert (or url ";; URL: https://package-lint.test/p?q#f\n"))
     (insert (or commentary ";;; Commentary:\n;; A test package, for testing.\n"))
     (insert contents)
-    (insert "\n" (or provide "(provide 'test)\n"))
-    (insert (or footer "\n\n;;; test.el ends here\n"))
-    (let ((buffer-file-name "test.el"))
+    (insert "\n" (or provide (format "(provide '%s)\n" featurename)))
+    (insert (or footer (format "\n\n;;; %s.el ends here\n" featurename)))
+    (let ((buffer-file-name (format "%s.el" featurename)))
       (package-lint-buffer))))
 
 (ert-deftest package-lint-test-reserved-keybindings ()
@@ -145,7 +152,7 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-warn-no-standard-keyword ()
   (should
    (equal
-    '((6 1 warning "You should include standard keywords: see the variable `finder-known-keywords'."))
+    '((6 3 warning "You should include standard keywords: see the variable `finder-known-keywords'."))
     (package-lint-test--run ";; Keywords: foo"))))
 
 (ert-deftest package-lint-test-no-warning-if-at-least-one-standard-keyword ()
@@ -155,15 +162,15 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-error-if-no-url ()
   (should
    (equal
-    '((1 1 error "Package should have a Homepage or URL header."))
+    '((1 0 error "Package should have a Homepage or URL header."))
     (package-lint-test--run "" nil nil nil nil nil "")))
   (should
    (equal
-    '((3 9 error "Package URLs should be a single HTTPS or HTTP URL."))
+    '((3 8 error "Package URLs should be a single HTTPS or HTTP URL."))
     (package-lint-test--run "" nil nil nil nil nil ";; URL: not a URL\n")))
   (should
    (equal
-    '((3 9 error "Package URLs should be a single HTTPS or HTTP URL."))
+    '((3 8 error "Package URLs should be a single HTTPS or HTTP URL."))
     (package-lint-test--run "" nil nil nil nil nil
                             ";; URL: git://test/test.git\n"))))
 
@@ -174,13 +181,13 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-warn-invalid-version ()
   (should
    (member
-    '(2 21 warning "\"invalid\" is not a valid version. MELPA will handle this, but other archives will not.")
+    '(2 20 warning "\"invalid\" is not a valid version. MELPA will handle this, but other archives will not.")
     (package-lint-test--run "" nil ";; Package-Version: invalid\n"))))
 
 (ert-deftest package-lint-test-warn-no-version ()
   (should
    (member
-    '(1 1 warning "\"Version:\" or \"Package-Version:\" header is missing. MELPA will handle this, but other archives will not.")
+    '(1 0 warning "\"Version:\" or \"Package-Version:\" header is missing. MELPA will handle this, but other archives will not.")
     (package-lint-test--run ";; Package-Requires: ((example \"0\"))" nil ""))))
 
 (ert-deftest package-lint-test-accept-valid-version ()
@@ -189,7 +196,7 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-error-lexical-binding-not-at-end ()
   (should
    (equal
-    '((1 1 error "`lexical-binding' must be set in the first line."))
+    '((1 0 error "`lexical-binding' must be set in the first line."))
     (package-lint-test--run
      ";; Local Variables:
 ;; lexical-binding: t
@@ -198,7 +205,7 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-warn-lexical-binding-without-emacs-24-dep ()
   (should
    (equal
-    '((1 28 warning "You should depend on (emacs \"24\") if you need lexical-binding."))
+    '((1 27 warning "You should depend on (emacs \"24\") if you need lexical-binding."))
     (package-lint-test--run
      ""
      ";;; test.el --- A test -*- lexical-binding: t -*-\n"))))
@@ -214,21 +221,22 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-warn-empty-summary ()
   (should
    (equal
-    '((1 1 warning "Package should have a non-empty summary."))
+    '((1 0 warning "Package should have a non-empty summary."))
     (package-lint-test--run "" ";;; test.el ---\n"))))
 
 (ert-deftest package-lint-test-warn-too-long-summary ()
   (should
    (equal
-    '((1 1 warning "The package summary is too long. It should be at most 50 characters."))
+    '((1 0 warning "The package summary is too long. It should be at most 60 characters.")
+      (1 0 warning "The package summary should start with an uppercase letter or a digit."))
     (package-lint-test--run
      ""
-     ";;; test.el --- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"))))
+     ";;; test.el --- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"))))
 
 (ert-deftest package-lint-test-warn-emacs-in-summary ()
   (should
    (equal
-    '((1 1 warning "Including \"Emacs\" in the package description is usually redundant."))
+    '((1 0 warning "Including \"Emacs\" in the package summary is usually redundant."))
     (package-lint-test--run "" ";;; test.el --- A package for Emacs\n"))))
 
 (ert-deftest package-lint-test-accept-emacs-lisp-in-summary ()
@@ -240,17 +248,17 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-error-invalid-dependency ()
   (should
    (member
-    '(6 1 error "Expected (package-name \"version-num\"), but found invalid.")
+    '(6 0 error "Expected (package-name \"version-num\"), but found invalid.")
     (package-lint-test--run ";; Package-Requires: (invalid)")))
   (should
    (member
-    '(6 24 error "\"invalid\" is not a valid version string: see `version-to-list'.")
+    '(6 23 error "\"invalid\" is not a valid version string: see `version-to-list'.")
     (package-lint-test--run ";; Package-Requires: ((package-lint \"invalid\"))"))))
 
 (ert-deftest package-lint-test-error-emacs-23-dep ()
   (should
    (equal
-    '((6 24 error "You can only depend on Emacs version 24 or greater: package.el for Emacs 23 does not support the \"emacs\" pseudopackage."))
+    '((6 23 error "You can only depend on Emacs version 24 or greater: package.el for Emacs 23 does not support the \"emacs\" pseudopackage."))
     (package-lint-test--run ";; Package-Requires: ((emacs \"23\"))"))))
 
 (ert-deftest package-lint-test-accept-emacs-24+-dep ()
@@ -260,19 +268,19 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-error-uninstallable-dep ()
   (should
    (equal
-    '((6 24 error "Package example-nonexistent-package is not installable."))
+    '((6 23 error "Package example-nonexistent-package is not installable."))
     (package-lint-test--run ";; Package-Requires: ((example-nonexistent-package \"1\"))"))))
 
 (ert-deftest package-lint-test-warn-snapshot-dep ()
   (should
    (equal
-    '((6 24 warning "Use a non-snapshot version number for dependency on \"package-lint\" if possible."))
+    '((6 23 warning "Use a non-snapshot version number for dependency on \"package-lint\" if possible."))
     (package-lint-test--run ";; Package-Requires: ((package-lint \"20160101.1234\"))"))))
 
 (ert-deftest package-lint-test-warn-unversioned-dep ()
   (should
    (equal
-    '((6 24 warning "Use a properly versioned dependency on \"package-lint\" if possible."))
+    '((6 23 warning "Use a properly versioned dependency on \"package-lint\" if possible."))
     (package-lint-test--run ";; Package-Requires: ((package-lint \"0\"))"))))
 
 (ert-deftest package-lint-test-warn-dependency-too-high ()
@@ -280,7 +288,7 @@ it's nil, the default is used."
     (package-lint-test-add-package-lint-foobar-to-archive '(0 5 0))
     (should
      (equal
-      '((6 24 warning "Version dependency for package-lint-foobar appears too high: try 0.5.0"))
+      '((6 23 warning "Version dependency for package-lint-foobar appears too high: try 0.5.0"))
       (package-lint-test--run ";; Package-Requires: ((package-lint-foobar \"0.6.0\"))")))))
 
 (ert-deftest package-lint-test-dont-warn-dependency-too-high-1 ()
@@ -307,7 +315,7 @@ it's nil, the default is used."
 (ert-deftest package-lint-test-error-cl-lib-1.0-dep ()
   (should
    (member
-    '(6 24 error "Depend on the latest 0.x version of cl-lib rather than on version \"(1)\".
+    '(6 23 error "Depend on the latest 0.x version of cl-lib rather than on version \"(1)\".
 Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled.")
     (package-lint-test--run ";; Package-Requires: ((cl-lib \"1\"))"))))
 
@@ -318,16 +326,23 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
 (ert-deftest package-lint-test-error-new-functions ()
   (should
    (equal
-    '((6 1 error "You should depend on (emacs \"25\") if you need `when-let'."))
+    '((6 1 error "You should depend on (emacs \"25.1\") if you need `when-let'."))
     (package-lint-test--run
      "(when-let ((foo (bar))) (message \"ok\"))"))))
 
+(ert-deftest package-lint-test-error-new-functions-as-quote ()
+  (should
+   (equal
+    '((6 20 error "You should depend on (emacs \"24.1\") if you need `window-resize'."))
+    (package-lint-test--run
+     "(defconst test-fn #'window-resize)"))))
+
 (ert-deftest package-lint-test-accept-new-functions-with-dep ()
   (should
    (equal
     '()
     (package-lint-test--run
-     ";; Package-Requires: ((emacs \"25\"))
+     ";; Package-Requires: ((emacs \"25.1\"))
 \(when-let ((foo (bar))) (message \"ok\"))"))))
 
 (ert-deftest package-lint-test-accept-new-functions-with-fboundp ()
@@ -339,11 +354,34 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
     (when-let blah)
   (bloop))"))))
 
+(ert-deftest package-lint-test-error-new-backported-functions ()
+  (should
+   (equal
+    '((6 1 error "You should depend on (emacs \"25.1\") or the seq package if you need `seq-length'."))
+    (package-lint-test--run
+     "(seq-length '(foo))"))))
+
+(ert-deftest package-lint-test-accepts-new-backported-functions-with-emacs-dep ()
+  (should
+   (equal
+    '()
+    (package-lint-test--run
+     ";; Package-Requires: ((emacs \"25.1\"))
+\(seq-length '(foo))"))))
+
+(ert-deftest package-lint-test-accepts-new-backported-functions-with-backport-dep ()
+  (should
+   (equal
+    '()
+    (package-lint-test--run
+     ";; Package-Requires: ((seq \"1\"))
+\(seq-length '(foo))"))))
+
 (ert-deftest package-lint-test-error-nonstandard-symbol-separator ()
   (should
    (equal
-    '((6 1 error "`test-thing/bar' contains a non-standard separator `/', use hyphens instead (see Elisp Coding Conventions).")
-      (7 1 error "`test-thing:bar' contains a non-standard separator `:', use hyphens instead (see Elisp Coding Conventions)."))
+    '((6 0 error "`test-thing/bar' contains a non-standard separator `/', use hyphens instead (see Elisp Coding Conventions).")
+      (7 0 error "`test-thing:bar' contains a non-standard separator `:', use hyphens instead (see Elisp Coding Conventions)."))
     (package-lint-test--run
      "(defun test-thing/bar () t)\n(defun test-thing:bar () nil)")))
   ;; But accept /= when at the end.
@@ -352,11 +390,11 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
 (ert-deftest package-lint-test-error-unprefixed-definitions ()
   (should
    (equal
-    '((6 1 error "\"foo\" doesn't start with package's prefix \"test\"."))
+    '((6 0 error "\"foo\" doesn't start with package's prefix \"test\"."))
     (package-lint-test--run "(defun foo ())")))
   (should
    (equal
-    '((6 1 error "\"global-testfoo-mode\" doesn't start with package's prefix \"test\"."))
+    '((6 0 error "\"global-testfoo-mode\" doesn't start with package's prefix \"test\"."))
     (package-lint-test--run "(define-globalized-minor-mode global-testfoo-mode ignore ignore :require 'test)"))))
 
 (ert-deftest package-lint-test-accept-prefixed-definitions ()
@@ -374,12 +412,38 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
 (ert-deftest package-lint-test-error-new-libraries ()
   (should
    (equal
-    '((6 10 error "You should depend on (emacs \"24.4\") if you need `nadvice'."))
-    (package-lint-test--run "(require 'nadvice)"))))
+    '((6 10 error "You should depend on (emacs \"24.4\") if you need `subr-x'."))
+    (package-lint-test--run "(require 'subr-x)"))))
+
+(ert-deftest package-lint-test-error-removed-libraries ()
+  (should
+   (equal
+    '((6 10 error "The `awk-mode' library was removed in Emacs version 26.1."))
+    (package-lint-test--run "(require 'awk-mode)"))))
+
+(ert-deftest package-lint-test-error-removed-functions ()
+  (should
+   (equal
+    '((6 1 error "`spell-buffer' was removed in Emacs version 26.1."))
+    (package-lint-test--run "(spell-buffer)"))))
 
 (ert-deftest package-lint-test-accept-new-libraries-with-dep ()
   (should (equal '() (package-lint-test--run
-                      ";; Package-Requires: ((emacs \"24.4\"))\n(require 'nadvice)"))))
+                      ";; Package-Requires: ((emacs \"24.4\"))\n(require 'subr-x)"))))
+
+(ert-deftest package-lint-test-error-new-backported-libraries ()
+  (should
+   (equal
+    '((6 10 error "You should depend on (emacs \"25.1\") or the seq package if you need `seq'."))
+    (package-lint-test--run "(require 'seq)"))))
+
+(ert-deftest package-lint-test-accept-new-backported-libraries-with-emacs-dep ()
+  (should (equal '() (package-lint-test--run
+                      ";; Package-Requires: ((emacs \"25.1\"))\n(require 'seq)"))))
+
+(ert-deftest package-lint-test-accept-new-backported-libraries-with-backport-dep ()
+  (should (equal '() (package-lint-test--run
+                      ";; Package-Requires: ((seq \"1\"))\n(require 'seq)"))))
 
 (ert-deftest package-lint-test-accept-new-libraries-with-optional-require ()
   (should (equal '() (package-lint-test--run "(require 'nadvice nil t)"))))
@@ -406,25 +470,29 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
 (ert-deftest package-lint-test-error-unmatched-first-and-last-lines ()
   (should
    (member
-    '(1 1 error "package.el cannot parse this buffer: Search failed: \";;; test.el ends here\"")
+    '(1 0 error "package.el cannot parse this buffer: Search failed: \";;; test.el ends here\"")
     (package-lint-test--run "" nil nil "\n\n;;; Test.el ends here\n"))))
 
 (ert-deftest package-lint-test-error-missing-provide-form ()
   (should
    (equal
-    '((1 1 error "There is no (provide 'test) form."))
+    '((1 0 error "There is no (provide 'test) form."))
     (package-lint-test--run "" nil nil nil ""))))
 
 (ert-deftest package-lint-test-error-mismatched-provide-form ()
   (should
    (equal
-    '((1 1 error "There is no (provide 'test) form."))
+    '((1 0 error "There is no (provide 'test) form."))
     (package-lint-test--run "" nil nil nil "(provide 'blargh)"))))
 
+(ert-deftest package-lint-test-accept-provide-me ()
+  (should
+   (equal '() (package-lint-test--run "" nil nil nil "(provide-me)"))))
+
 (ert-deftest package-lint-test-error-no-commentary ()
   (should
    (equal
-    '((1 1 error "Package should have a ;;; Commentary section."))
+    '((1 0 error "Package should have a ;;; Commentary section."))
     (package-lint-test--run "" nil nil nil nil "\n"))))
 
 (ert-deftest package-lint-test-error-empty-commentary ()
@@ -440,7 +508,8 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
   (should (equal '() (package-lint-test--run "   (  defadvice \t\n\n foo (before ignore))"))))
 
 (ert-deftest package-lint-test-accept-unprefixed-cl-defmethod ()
-  (should (equal '() (package-lint-test--run "(cl-defmethod foo ()"))))
+  (should (equal '() (package-lint-test--run ";; Package-Requires: ((emacs \"25.1\"))
+\(cl-defmethod foo ()"))))
 
 (ert-deftest package-lint-test-minor-mode-global-t ()
   (should
@@ -469,6 +538,16 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
     '((6 0 error "Global minor modes should be autoloaded or, rarely, `:require' their defining file (i.e. \":require 'test\"), to support the customization variable of the same name."))
     (package-lint-test--run "(define-globalized-minor-mode test-mode ignore ignore :require 'blargh)"))))
 
+(ert-deftest package-lint-test-warning-eval-after-load ()
+  (should
+   (equal
+    '((6 1 warning "`eval-after-load' is for use in configurations, and should rarely be used in packages."))
+    (package-lint-test--run "(eval-after-load 'foobar\n body)")))
+  (should
+   (equal
+    '((7 1 warning "`with-eval-after-load' is for use in configurations, and should rarely be used in packages."))
+    (package-lint-test--run ";; Package-Requires: ((emacs \"24.4\"))\n(with-eval-after-load 'foobar\n body)"))))
+
 (ert-deftest package-lint-test-error-defgroup-name ()
   (should
    (equal
@@ -489,5 +568,72 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
     '((6 0 error "Customization groups should specify a parent via `:group'."))
     (package-lint-test--run "(defgroup test nil \"\")"))))
 
+(ert-deftest package-lint-test-error-defalias-name ()
+  (should
+   (equal
+    '((6 0 error "Aliases should start with the package's prefix \"test\"."))
+    (package-lint-test--run "(defalias 'foobar 'string-equal)")))
+  (should
+   (equal
+    '((6 0 error "Aliases should start with the package's prefix \"test\"."))
+    (package-lint-test--run "(defvaralias 'foobar 'string-equal)")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(defalias 'test-foobar 'string-equal)")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(defvaralias 'test-foobar 'string-equal)"))))
+
+(ert-deftest package-lint-test-error-format-string ()
+  (should
+   (equal
+    '((6 0 error "You should depend on (emacs \"26.1\") if you need format field numbers."))
+    (package-lint-test--run "(format \"%1$s\" \"foo\")")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(format \"%s\" \"foo\")")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(format \"%%1$s\" \"foo\")")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run ";; Package-Requires: ((emacs \"26.1\"))
+\(format \"%1$s\" \"foo\")"))))
+
+(ert-deftest package-lint-test-accept-quasiquoted-defalias ()
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(defalias ',foo 'bar)"))))
+
+(ert-deftest package-lint-test-no-emacs-in-package-name ()
+  (should
+   (equal
+    '((1 0 warning "The word \"emacs\" is redundant in Emacs package names."))
+    (package-lint-test--run "" nil nil nil nil nil nil "emacs-package"))))
+
+(ert-deftest package-lint-test-warn-about-lonely-parens ()
+  (should
+   (equal
+    '((7 0 warning "Closing parens should not be wrapped onto new lines."))
+    (package-lint-test--run "(hello\n)")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(hello\n 'world)")))
+  (should
+   (equal
+    '((7 5 warning "Closing parens should not be wrapped onto new lines."))
+    (package-lint-test--run "(foo (hello\n     ) bar)")))
+  (should
+   (equal
+    '((7 2 warning "Closing parens should not be wrapped onto new lines."))
+    (package-lint-test--run "(hello\n  )    ; foo"))))
+
 (provide 'package-lint-test)
 ;;; package-lint-test.el ends here
